#!/usr/bin/perl -w

# register.cgi: enable a committee member to enter reviews for
#               Faculty applicants and select notification preference
#
# Copyright (C) 2000-2003 Filippo Menczer and the University of Iowa
# Copyright (C) 2003-2004 Filippo Menczer and Indiana University 
#
#    This file is part of the Recruit system
#
#    Recruit is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Recruit is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Recruit; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# CONVENTIONS: 
# - admin password (encrypted) is stored in file 'passwd'
# - committee info is stored in database 'committee.DB'
# - the value of a committee entry is "email $; X $; area $; area ..." 
#   where X=1 for email notification, X=0 o.w.

$|=1;
use strict;
use CGI::Carp qw(fatalsToBrowser);
use CGI qw(:standard);
use DB_File;

# PARAMETERS OF THE SEARCH
use lib qw(.);
use params ();
use vars qw(%global);
*global = \%params::global;
my $dept = $global{dept};
my $passwdflag = $global{passwdflag};
my $arealist = $global{areas};

# check password against stored encrypted one (make sure file 'passwd' is there!)
# NB: if $passwdflag==0, then the check is passed automatically without any password!
# NB: hope salt is OK ...
sub check ($) {
	return 1 unless $passwdflag; ### NO AUTHENTICATION!!!!
	my $clear = shift;
	return 0 unless defined($clear);
	open(PW, "passwd") || die "passwd file not found: $!";
	my $encoded = <PW>;
	close PW;
	chomp($encoded);
	if (crypt($clear, $encoded) eq $encoded) {
		return 1;
	} else {
		return 0;
	}
}

# get next ID for committee hash
sub newID (%) {
	my %h = @_;
	my $max = -1;
	while (my($k,$v) = each %h) {
		$max = $k if ($max < $k);
	}
	return ($max + 1);
}

# MAIN
print header, start_html(-title=>"$dept Recruit"), 
	 h1("Register as committee member to review faculty applicants in $dept");
	 
if (!param() && !check(cookie('cypher'))) { # stage 0, enter passwd

    print start_form();
	 print p("Password:", password_field('secret','',50), submit("Submit!"));
	 print hidden('stage',1);
    print end_form();
	 
} elsif ((!param() && check(cookie('cypher'))) || (param('stage') == 1 && check(param('secret')))) { # stage 1

    print start_form();
	 print p("To register enter your email, area profile, and notification preference, then select 'Add New Reviewer'. ",
		"To update your email, area profile, and/or notification preference select your current email from the list, enter your email (if changed), profile, and notification preference, then select 'Update Profile'. ",
		"To remove a reviewer select the email from the list and 'Remove Reviewer'.");
	 
	 # Show list for deleting
	 my %comm;
	 tie(%comm, 'DB_File', "committee.DB") || die "Cannot open candidate DB_File: $!";
	 my %labels;
	 foreach my $k (keys %comm) {
	 	my @profile = split(/$;/, $comm{$k});
	 	$labels{$k} = "$k. $profile[0]";
 	 }
	 untie %comm;
	 my @revlist = sort(keys %labels);
	 print scrolling_list('reviewer', \@revlist, [], 10, 'false', \%labels);
	
	 # Enter email (id) for adding
	 print p("Email:", textfield('name','',50));
	 
	 # Get reviewer's filter preferences
	 print p("Your areas of interest: ", br, checkbox_group('areafilter', $arealist, $arealist, 'true'));

	 # Check box for email notification
	 print checkbox(-name=>"notify", -value=>1, -label=>'Notify me of new applications (if email supported)');
	 
	 # decide what to do
	 print p, radio_group('option', ['Add New Reviewer', 'Update Profile', 'Remove Reviewer'],
		  'Add New Reviewer', 'true');

	# save stage and passwd in hidden field
	my $passwd; 
	if ($passwdflag) {
		$passwd = param('secret');
	} else {
		$passwd = ''; # does not matter what as long as defined
	}
	print hidden('secret', $passwd);
	param('stage',2);
 	print hidden('stage',2);

	print p(submit("Submit Request!"));
    print end_form();
    
} elsif (param('stage') == 2 && check(param('secret'))) { # stage 2

	 # Get reviewer name, assign new ID	
	 my %comm;
	 tie(%comm, 'DB_File', "committee.DB") || die "Cannot open candidate DB_File: $!";
	 if (param('option') eq 'Add New Reviewer') {
		 my $id = newID(%comm);
		 my @profile = (param('name'), param('notify')? '1' : '0', param('areafilter'));
		 $comm{$id} = join("$;", @profile);
		 print p("Thanks for registering as a reviewer. Your ID is $id, your email is <strong>", param('name'), "</strong> and notification is", param('notify')? "on.":"off.");
		 print "You may now evaluate applications from the ", a({href=>"../index.cgi"}, "Recruit Home"), ".";
	 } elsif (param('option') eq 'Update Profile') {
		my @profile = split(/$;/, $comm{param('reviewer')});
		$profile[0] = param('name') if length(param('name'));
		@profile = ($profile[0], param('notify')? '1' : '0', param('areafilter'));
		$comm{param('reviewer')} = join("$;", @profile);
		print p("Profile updated. Email notification for <strong>$profile[0]</strong> is ", param('notify')?"on.":"off.");
	 } elsif (param('option') eq 'Remove Reviewer') {
	 	 my @profile = split(/$;/, $comm{param('reviewer')});
	 	 delete $comm{param('reviewer')};
		 print p("<strong>$profile[0]</strong> has been removed from the list of reviewers.");
	 } else {
		  warn "Error: unrecognized option!";
	 }
	 untie %comm;

} else { # FAILURE STAGE

  	 print h2("Authentication failed.");
  	 
}
print hr, a({href=>"http://www.informatics.indiana.edu/fil/Recruit/", target=>"_new"}, 'Recruit'), " &copy; 2000-2004 ", a({href=>"http://www.informatics.indiana.edu/fil/", target=>"_new"}, 'Filippo Menczer'); 
print end_html;

